/************************************************************************************
**    SIMphoto is an aplication based on single image photogrammetry. It is useful
**    for creating and printing a photomap. The software is designed for close
**    range photogrammtery. Outputs are usable in the field of heritage preservation.
**    Copyright (C) 2009 David Cizek
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**    How to contact me: cizekdav@gmail.com
**
***********************************************************************************/

#include "maplayoutdialog.h"

MapLayoutDialog::MapLayoutDialog(QImage *photoMap, QRect *mapRect, double pixSize, QString filePath, QWidget *parent)
    : QDialog(parent)
{
    Qt::WindowFlags flags;
    flags = Qt::Dialog;
    flags |= Qt::WindowCloseButtonHint;

    mapFixed = false;
    pixelSize = pixSize;
    dir = filePath;

    map = new QImage(*photoMap);
    rect = new QRect(mapRect->x(), mapRect->y(), mapRect->width(), mapRect->height());
    image = new QImage(rect->width(), rect->height(), QImage::Format_ARGB32);

    w = 210;
    h = 297;

    moveTitleLabel = new QLabel(tr("Click into preview to move title."), this);
    moveDescrpLabel = new QLabel(tr("Click into preview to move description."), this);
    previewTitleLabel = new QLabel(tr("Paper A4: 210 x 297 mm"), this);
    titleLabel = new QLabel(tr("Title"), this);
    titleSizeLabel = new QLabel(tr("Text Size"), this);
    descriptionLabel = new QLabel(tr("Description"), this);
    descriptionSizeLabel = new QLabel(tr("Text Size"), this);
    pixelSizeLabel = new QLabel(tr("Pixel size: 1 pixel = ") + QString::number(pixSize, 'f', 4) + " m", this);
    paperLabel = new QLabel(tr("Paper"), this);
    orientationLabel = new QLabel(tr("Orientation"), this);
    scaleLabel = new QLabel(tr("Scale 1 :"), this);

    moveTitleLabel->setEnabled(false);
    moveDescrpLabel->setEnabled(false);
    titleLabel->setEnabled(false);
    titleSizeLabel->setEnabled(false);
    descriptionLabel->setEnabled(false);
    descriptionSizeLabel->setEnabled(false);

    mapRadioButton = new QRadioButton(tr("Insert PhotoMap"), this);
    titleRadioButton = new QRadioButton(tr("Insert Title"), this);
    descriptionRadioButton = new QRadioButton(tr("Insert Description"), this);

    mapRadioButton->setChecked(true);

    paperComboBox = new QComboBox(this);
    paperComboBox->addItem("A4: 210 x 297 mm");
    paperComboBox->addItem("A3: 297 x 420 mm");
    paperComboBox->addItem("A2: 420 x 594 mm");

    orientationComboBox = new QComboBox(this);
    orientationComboBox->addItem(tr("portrait"));
    orientationComboBox->addItem(tr("landscape"));

    printScaleBox = new QSpinBox(this);
    printScaleBox->setRange(1, 50000);
    printScaleBox->setValue(50);

    titleSizeBox = new QSpinBox(this);
    descriptionSizeBox = new QSpinBox(this);

    titleSizeBox->setRange(10, 50);
    descriptionSizeBox->setRange(5, 50);

    titleSizeBox->setEnabled(false);
    descriptionSizeBox->setEnabled(false);

    titleLineEdit = new QLineEdit(this);
    descriptionLineEdit = new QLineEdit(this);

    titleLineEdit->setEnabled(false);
    descriptionLineEdit->setEnabled(false);

    backButton = new QPushButton(tr("<< Back"), this);
    printButton = new QPushButton(tr("Print To PDF"), this);
    printButton->setEnabled(false);

    QSpacerItem *hSpacer = new QSpacerItem(50, 8, QSizePolicy::Expanding, QSizePolicy::Minimum);
    QSpacerItem *vSpacer1 = new QSpacerItem(8, 10, QSizePolicy::Minimum, QSizePolicy::Expanding);
    QSpacerItem *vSpacer2 = new QSpacerItem(8, 10, QSizePolicy::Minimum, QSizePolicy::Expanding);
    QSpacerItem *vSpacer3 = new QSpacerItem(8, 10, QSizePolicy::Minimum, QSizePolicy::Expanding);

    paper = new QImage(2480, 3508, QImage::Format_ARGB32);
    QRgb white = qRgba(255, 255, 255, 255);
    paper->fill(white);

    mapItem = new QGraphicsPixmapItem;
    titleItem = new QGraphicsTextItem;
    descriptionItem = new QGraphicsTextItem;
    pixmapItem = new QGraphicsPixmapItem(QPixmap::fromImage(paper->scaled(500, 500, Qt::KeepAspectRatio, Qt::FastTransformation)));
    rectItem = new QGraphicsRectItem(0, 0, (int)(rect->width() * pixelSize * 500000 / printScaleBox->value() / 297),
                                           (int)(rect->height() * pixelSize * 500000 / printScaleBox->value() / 297));
    backgroundItem = new QGraphicsRectItem(0, 0, 500, 500);

    QBrush brush(QColor(0, 0, 0, 255), Qt::SolidPattern);
    backgroundItem->setBrush(brush);

    pixmapItem->setPos((500 - pixmapItem->boundingRect().width()) / 2, 0);
    titleItem->setPos(80, 10);
    descriptionItem->setPos(80, 470);

    brush.setColor(QColor(170, 0, 0, 100));
    rectItem->setPen(QPen(QColor(170, 0, 0), 2, Qt::SolidLine,Qt::FlatCap, Qt::MiterJoin));
    rectItem->setBrush(brush);

    backgroundItem->setZValue(0);
    pixmapItem->setZValue(1);
    mapItem->setZValue(2);
    descriptionItem->setZValue(3);
    titleItem->setZValue(4);
    rectItem->setZValue(5);

    graphicsView = new QGraphicsView(this);
    graphicsScene = new QGraphicsScene(this);
    graphicsView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    graphicsView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    graphicsView->setScene(graphicsScene);

    graphicsScene->setSceneRect(0, 0, 500, 500);

    graphicsScene->addItem(backgroundItem);
    graphicsScene->addItem(pixmapItem);
    graphicsScene->addItem(rectItem);
    graphicsScene->addItem(mapItem);
    graphicsScene->addItem(titleItem);
    graphicsScene->addItem(descriptionItem);

    graphicsView->viewport()->installEventFilter(this);
    graphicsView->viewport()->setMouseTracking(true);
    graphicsView->setFixedSize(500, 500);
    graphicsView->viewport()->setCursor(Qt::CrossCursor);

    QVBoxLayout *titleVLayout = new QVBoxLayout;
    QVBoxLayout *descrpVLayout = new QVBoxLayout;
    QVBoxLayout *vToolsLayout = new QVBoxLayout;
    QVBoxLayout *mainLayout = new QVBoxLayout;
    QVBoxLayout *radioButtonLayout = new QVBoxLayout;
    QHBoxLayout *hButtonsLayout = new QHBoxLayout;
    QGridLayout *gridLayout = new QGridLayout;
    QGridLayout *paperLayout = new QGridLayout;
    QGridLayout *titleLayout = new QGridLayout;
    QGridLayout *descriptionLayout = new QGridLayout;

    paperLayout->addWidget(paperLabel, 0, 0);
    paperLayout->addWidget(paperComboBox, 0, 1);
    paperLayout->addWidget(orientationLabel, 1, 0);
    paperLayout->addWidget(orientationComboBox, 1, 1);
    paperLayout->addWidget(scaleLabel, 2, 0);
    paperLayout->addWidget(printScaleBox, 2, 1);

    titleLayout->addWidget(titleLabel, 0, 0);
    titleLayout->addWidget(titleLineEdit, 0, 1);
    titleLayout->addWidget(titleSizeLabel, 1, 0);
    titleLayout->addWidget(titleSizeBox, 1, 1);

    titleVLayout->addWidget(moveTitleLabel);
    titleVLayout->addLayout(titleLayout);

    descriptionLayout->addWidget(descriptionLabel, 0, 0);
    descriptionLayout->addWidget(descriptionLineEdit, 0, 1);
    descriptionLayout->addWidget(descriptionSizeLabel, 1, 0);
    descriptionLayout->addWidget(descriptionSizeBox, 1, 1);

    descrpVLayout->addWidget(moveDescrpLabel);
    descrpVLayout->addLayout(descriptionLayout);

    radioButtonLayout->addWidget(mapRadioButton);
    radioButtonLayout->addWidget(titleRadioButton);
    radioButtonLayout->addWidget(descriptionRadioButton);

    vToolsLayout->addWidget(pixelSizeLabel);
    vToolsLayout->addLayout(paperLayout);
    vToolsLayout->addItem(vSpacer1);
    vToolsLayout->addLayout(titleVLayout);
    vToolsLayout->addItem(vSpacer2);
    vToolsLayout->addLayout(descrpVLayout);
    vToolsLayout->addItem(vSpacer3);
    vToolsLayout->addLayout(radioButtonLayout);

    gridLayout->addWidget(previewTitleLabel, 0, 0);
    gridLayout->addWidget(graphicsView, 1, 0);
    gridLayout->addLayout(vToolsLayout, 1, 1);

    hButtonsLayout->addWidget(backButton);
    hButtonsLayout->addItem(hSpacer);
    hButtonsLayout->addWidget(printButton);

    mainLayout->addLayout(gridLayout);
    mainLayout->addLayout(hButtonsLayout);

    connect(paperComboBox, SIGNAL(currentIndexChanged(QString)), this, SLOT(showPaper()));
    connect(orientationComboBox, SIGNAL(currentIndexChanged(QString)), this, SLOT(showPaper()));
    connect(printScaleBox, SIGNAL(valueChanged(int)), this, SLOT(showPaper()));
    connect(mapRadioButton, SIGNAL(clicked()), this, SLOT(changeTool()));
    connect(titleRadioButton, SIGNAL(clicked()), this, SLOT(changeTool()));
    connect(descriptionRadioButton, SIGNAL(clicked()), this, SLOT(changeTool()));
    connect(titleSizeBox, SIGNAL(valueChanged(int)), this, SLOT(changeTextSize()));
    connect(titleLineEdit, SIGNAL(textChanged(QString)), this, SLOT(changeTextSize()));
    connect(descriptionSizeBox, SIGNAL(valueChanged(int)), this, SLOT(changeTextSize()));
    connect(descriptionLineEdit, SIGNAL(textChanged(QString)), this, SLOT(changeTextSize()));
    connect(printButton, SIGNAL(clicked()), this, SLOT(drawPrintPixmap()));

    setLayout(mainLayout);
    setWindowFlags(flags);
    setWindowTitle(tr("Map Layout"));
    setModal(true);
}

//return size of raster that represents paper
QSize MapLayoutDialog::paperSize()
{
    QSize size;

    previewTitleLabel->setText("Paper " + paperComboBox->currentText());
    mapFixed = false;
    printButton->setEnabled(false);
    mapItem->hide();

    QBrush brush(QColor(170, 0, 0, 100), Qt::SolidPattern);
    rectItem->setPen(QPen(QColor(170, 0, 0), 2, Qt::SolidLine,Qt::FlatCap, Qt::MiterJoin));
    rectItem->setBrush(brush);

    //paper size
    if(paperComboBox->currentIndex() == 0)
    {
        w = 210;
        h = 297;
    }
    else if(paperComboBox->currentIndex() == 1)
    {
        w = 297;
        h = 420;
    }
    else
    {
        w = 420;
        h = 594;
    }

    if(orientationComboBox->currentIndex() == 0)
    {
        //resolution: 300 dpi
        size.setWidth((int)round(w * 300 / 25.4));
        size.setHeight((int)round(h * 300 / 25.4));

        titleItem->setPos(80, 10);
        descriptionItem->setPos(80, 470);
    }
    else
    {
        size.setWidth((int)round(h * 300 / 25.4));
        size.setHeight((int)round(w * 300 / 25.4));

        titleItem->setPos(10, 80);
        descriptionItem->setPos(10, 405);
    }

    return size;
}

//paper preview
void MapLayoutDialog::showPaper()
{
    paper->~QImage();
    paper = new QImage(paperSize(), QImage::Format_ARGB32);
    QRgb white = qRgba(255, 255, 255, 255);
    paper->fill(white);

    int mapW, mapH;

    //getting size of photomap in scale of preview (width or height of preview is 500 pixels)
    mapW = (int)(rect->width() * pixelSize * 1000 * 500 / printScaleBox->value() / h);
    mapH = (int)(rect->height() * pixelSize * 1000 * 500 / printScaleBox->value() / h);

    rectItem->setRect(0, 0, mapW, mapH);

    graphicsScene->removeItem(pixmapItem);
    pixmapItem->~QGraphicsPixmapItem();
    pixmapItem = new QGraphicsPixmapItem(QPixmap::fromImage(paper->scaled(500, 500, Qt::KeepAspectRatio, Qt::FastTransformation)));

    if(orientationComboBox->currentIndex() == 0)
    {
        pixmapItem->setPos((500 - pixmapItem->boundingRect().width()) / 2, 0);
    }
    else
    {
        pixmapItem->setPos(0, (500 - pixmapItem->boundingRect().height()) / 2);
    }
    pixmapItem->setZValue(1);

    graphicsScene->addItem(pixmapItem);
}

bool MapLayoutDialog::eventFilter(QObject *obj, QEvent *event)
{
    if(obj == graphicsView->viewport())
    {
        if(event->type() == QEvent::Enter && mapRadioButton->isChecked())
        {
            QBrush brush(QColor(170, 0, 0, 100), Qt::SolidPattern);
            rectItem->setPen(QPen(QColor(170, 0, 0), 2, Qt::SolidLine,Qt::FlatCap, Qt::MiterJoin));
            rectItem->setBrush(brush);

            mapFixed = false;
        }
        if(event->type() == QEvent::MouseButtonPress)
        {
            QMouseEvent *pressEvent = static_cast<QMouseEvent*>(event);

            if(pressEvent->button() == Qt::LeftButton && mapRadioButton->isChecked())
            {
                rectItem->setPos(pressEvent->x(), pressEvent->y());

                QBrush brush(QColor(255, 201, 14, 100), Qt::SolidPattern);
                rectItem->setPen(QPen(QColor(255, 201, 14), 2, Qt::SolidLine,Qt::FlatCap, Qt::MiterJoin));
                rectItem->setBrush(brush);

                addMapItem(pressEvent->x(), pressEvent->y());
                mapItem->show();

                mapFixed = true;
                printButton->setEnabled(true);
            }

            if(pressEvent->button() == Qt::LeftButton && titleRadioButton->isChecked())
            {
                addTitleItem(pressEvent->x(), pressEvent->y());
            }

            if(pressEvent->button() == Qt::LeftButton && descriptionRadioButton->isChecked())
            {
                addDescriptionItem(pressEvent->x(), pressEvent->y());
            }

            return true;
        }
        if(event->type() == QEvent::MouseMove && !mapFixed  && mapRadioButton->isChecked())
        {
            QMouseEvent *moveEvent = static_cast<QMouseEvent*>(event);

            rectItem->setPos(moveEvent->x(), moveEvent->y());

            return true;
        }
        else
        {
            return false;
        }
    }
    else
    {
         return QDialog::eventFilter(obj, event);
    }
}

void MapLayoutDialog::addMapItem(int x, int y)
{
    QPainter *painter = new QPainter(image);

    painter->setWindow(*rect);
    painter->drawPixmap(0, 0, QPixmap::fromImage(*map));

    mapItem->setPixmap(QPixmap::fromImage(image->scaled((int)rectItem->rect().width(), (int)rectItem->rect().height(), Qt::KeepAspectRatio, Qt::FastTransformation)));
    mapItem->setPos(x, y);

    painter->end();
}

void MapLayoutDialog::addTitleItem(int x, int y)
{
    titleItem->setPos(x, y);
    titleItem->setFont(QFont("Arial", titleSizeBox->value(), QFont::Bold));
    titleItem->setPlainText(titleLineEdit->text());
}

void MapLayoutDialog::addDescriptionItem(int x, int y)
{
    descriptionItem->setPos(x, y);
    descriptionItem->setFont(QFont("Arial", descriptionSizeBox->value(), QFont::Normal));
    descriptionItem->setPlainText(descriptionLineEdit->text());
}

void MapLayoutDialog::changeTextSize()
{
    if(titleRadioButton->isChecked())
        addTitleItem((int)titleItem->pos().x(), (int)titleItem->pos().y());

    if(descriptionRadioButton->isChecked())
        addDescriptionItem((int)descriptionItem->pos().x(), (int)descriptionItem->pos().y());
}

void MapLayoutDialog::changeTool()
{
    if(mapRadioButton->isChecked())
    {
        rectItem->show();

        pixelSizeLabel->setEnabled(true);
        paperLabel->setEnabled(true);
        orientationLabel->setEnabled(true);
        scaleLabel->setEnabled(true);
        paperComboBox->setEnabled(true);
        orientationComboBox->setEnabled(true);
        printScaleBox->setEnabled(true);

        moveTitleLabel->setEnabled(false);
        titleLabel->setEnabled(false);
        titleSizeLabel->setEnabled(false);
        titleSizeBox->setEnabled(false);
        titleLineEdit->setEnabled(false);

        moveDescrpLabel->setEnabled(false);
        descriptionLabel->setEnabled(false);
        descriptionSizeLabel->setEnabled(false);
        descriptionSizeBox->setEnabled(false);
        descriptionLineEdit->setEnabled(false);
    }

    if(titleRadioButton->isChecked())
    {
        rectItem->hide();

        pixelSizeLabel->setEnabled(false);
        paperLabel->setEnabled(false);
        orientationLabel->setEnabled(false);
        scaleLabel->setEnabled(false);
        paperComboBox->setEnabled(false);
        orientationComboBox->setEnabled(false);
        printScaleBox->setEnabled(false);

        moveTitleLabel->setEnabled(true);
        titleLabel->setEnabled(true);
        titleSizeLabel->setEnabled(true);
        titleSizeBox->setEnabled(true);
        titleLineEdit->setEnabled(true);

        moveDescrpLabel->setEnabled(false);
        descriptionLabel->setEnabled(false);
        descriptionSizeLabel->setEnabled(false);
        descriptionSizeBox->setEnabled(false);
        descriptionLineEdit->setEnabled(false);
    }

    if(descriptionRadioButton->isChecked())
    {
        rectItem->hide();

        pixelSizeLabel->setEnabled(false);
        paperLabel->setEnabled(false);
        orientationLabel->setEnabled(false);
        scaleLabel->setEnabled(false);
        paperComboBox->setEnabled(false);
        orientationComboBox->setEnabled(false);
        printScaleBox->setEnabled(false);

        moveTitleLabel->setEnabled(false);
        titleLabel->setEnabled(false);
        titleSizeLabel->setEnabled(false);
        titleSizeBox->setEnabled(false);
        titleLineEdit->setEnabled(false);

        moveDescrpLabel->setEnabled(true);
        descriptionLabel->setEnabled(true);
        descriptionSizeLabel->setEnabled(true);
        descriptionSizeBox->setEnabled(true);
        descriptionLineEdit->setEnabled(true);
    }
}

//creating print raster before exporting to PDF
void MapLayoutDialog::drawPrintPixmap()
{
    QPainter *painter = new QPainter(paper);
    int x, y;

    x = (int)((mapItem->pos().x() - pixmapItem->pos().x()) * h * 300 / 25.4 / 500);
    y = (int)((mapItem->pos().y() - pixmapItem->pos().y()) * h * 300 / 25.4 / 500);

    int itemW, itemH;

    if(orientationComboBox->currentIndex() == 0)
    {
        itemW = (int)(rect->width() * pixelSize * 1000 * paper->height() / printScaleBox->value() / h);
        itemH = (int)(rect->height() * pixelSize * 1000 * paper->height() / printScaleBox->value() / h);
    }
    else
    {
        itemW = (int)(rect->width() * pixelSize * 1000 * paper->width() / printScaleBox->value() / h);
        itemH = (int)(rect->height() * pixelSize * 1000 * paper->width() / printScaleBox->value() / h);
    }

    painter->drawPixmap(x, y, QPixmap::fromImage(image->scaled(itemW, itemH, Qt::KeepAspectRatio, Qt::SmoothTransformation)));

    x = (int)((titleItem->pos().x() - pixmapItem->pos().x()) * h * 300 / 25.4 / 500);
    y = (int)((titleItem->pos().y() - pixmapItem->pos().y()) * h * 300 / 25.4 / 500);

    itemW = (int)(titleItem->boundingRect().width() * h * 300 / 25.4 / 500);
    itemH = (int)(titleItem->boundingRect().height() * h * 300 / 25.4 / 500);

    painter->setPen(QColor(0, 0, 0));
    painter->setFont(QFont("Arial", (int)(titleSizeBox->value() * h * 300 / 25.4 / 500), QFont::Bold));
    painter->drawText(x, y, itemW, itemH, Qt::AlignCenter, titleLineEdit->text());

    x = (int)((descriptionItem->pos().x() - pixmapItem->pos().x()) * h * 300 / 25.4 / 500);
    y = (int)((descriptionItem->pos().y() - pixmapItem->pos().y()) * h * 300 / 25.4 / 500);

    itemW = (int)(descriptionItem->boundingRect().width() * h * 300 / 25.4 / 500);
    itemH = (int)(descriptionItem->boundingRect().height() * h * 300 / 25.4 / 500);

    painter->setFont(QFont("Arial", (int)(descriptionSizeBox->value() * h * 300 / 25.4 / 500), QFont::Normal));
    painter->drawText(x, y, itemW, itemH, Qt::AlignCenter, descriptionLineEdit->text());

    painter->end();
    delete painter;

    printImage();
}

//export to PDF
void MapLayoutDialog::printImage()
{
    QFileDialog saveImageDialog;
    QString fileName = saveImageDialog.getSaveFileName(this, tr("Export Image To PDF"),
                            dir, "PDF (*.pdf)");

    if(!fileName.isEmpty())
    {
        QPrinter printer;

        if(paperComboBox->currentIndex() == 0)
            printer.setPaperSize(QPrinter::A4);

        if(paperComboBox->currentIndex() == 1)
            printer.setPaperSize(QPrinter::A3);

        if(paperComboBox->currentIndex() == 2)
            printer.setPaperSize(QPrinter::A2);

        if(orientationComboBox->currentIndex() == 0)
            printer.setOrientation(QPrinter::Portrait);

        if(orientationComboBox->currentIndex() == 1)
            printer.setOrientation(QPrinter::Landscape);

        printer.setResolution(300);
        printer.setFullPage(true);
        printer.setOutputFormat(QPrinter::PdfFormat);
        printer.setOutputFileName(fileName);

        QPainter painter;
        painter.begin(&printer);

        if(painter.isActive())
        {
            painter.drawPixmap(0, 0, QPixmap::fromImage(*paper));
        }
        else
        {
            QMessageBox::information(this, tr("Warning"), tr("Image cannot be exported! Document is used by another aplication."));

            QPointF *titlePos = new QPointF(titleItem->pos().x(), titleItem->pos().y());
            QPointF *descriptionPos = new QPointF(descriptionItem->pos().x(), descriptionItem->pos().y());

            showPaper();
            mapItem->show();
            printButton->setEnabled(true);

            titleItem->setPos(*titlePos);
            descriptionItem->setPos(*descriptionPos);

            delete titlePos;
            delete descriptionPos;

            return;
        }
        painter.end();

        delete this;
    }
}

void MapLayoutDialog::closeEvent(QCloseEvent *closeEvent)
{
    delete this;
}

QPushButton* MapLayoutDialog::getBackButton()
{
    return backButton;
}
