/************************************************************************************
**    SIMphoto is an aplication based on single image photogrammetry. It is useful
**    for creating and printing a photomap. The software is designed for close
**    range photogrammtery. Outputs are usable in the field of heritage preservation.
**    Copyright (C) 2009 David Cizek
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**    How to contact me: cizekdav@gmail.com
**
***********************************************************************************/

#ifndef SIMPHOTO_H
#define SIMPHOTO_H

#include <QtGui>
#include "overview.h"
#include "marktool.h"
#include "cursorcolordialog.h"
#include "networkmethoddialog.h"
#include "memoryclass.h"
#include "distmethoddialog.h"
#include "measuretool.h"
#include "similartransformdialog.h"
#include "tableofpoints.h"
#include "scalebardialog.h"
#include "distanceinputdialog.h"
#include "pixelsizedialog.h"
#include "checkpointdialog.h"
#include "clipimagedialog.h"
#include "maplayoutdialog.h"
#include "distorsiondialog.h"
#include "aboutdialog.h"
#include <eigen2/Eigen/Core>
#include <eigen2/Eigen/lu>

USING_PART_OF_NAMESPACE_EIGEN

class SIMphoto : public QMainWindow
{
    Q_OBJECT

public slots:
    void openImage();
    void saveImage();
    void printImage();
    void exitApl();
    void undo();
    void doControlPointsMethodAction();
    void doNetworkMethodAction();
    void doDistMethodAction();
    void doSimilarTransAction();
    void doDistorsionRedAction();
    void doLoadTableAction();
    void doMeasureAction();
    void doScalebarAction();
    void doOverviewAction();
    void doCursorColorAction();
    void doZoom3xAction();
    void doZoom5xAction();
    void doNoZoomAction();
    void setFramePos();
    void closeMarkingTool();
    void startMarking();
    void setCrossCursor();
    void setCursorColor();
    void showAddedPoint();
    void erasePointFromOverview();
    void closeNetworkMethodDialog();
    void coordsFromNetwork();
    void closeDistMethodDialog();
    void getDistMethod();
    void setFirstPoint();
    void setSecondPoint();
    void closeMeasureAction();
    void showVerticalDaistance();
    void hideVerticalDistance();
    void closeSimilarTransDialog();
    void closeScalebarDialog();
    void finishScalebar();
    void getSimilarTransParameters();
    void getMethod();
    void getScaleFromDist();
    void closeDistInputDialog();
    void getSizeOfPixel();
    void closePixelSizeDialog();
    void checkInputData();
    void closeTableOfPoints();
    void saveTableData();
    void loadCoordinates();
    void setFocusOnPoint();
    void createMapLayoutDialog();
    void backToClipImage();
    void showResiduals();
    void closeDistortionDialog();
    void checkDistortionInput();
    void doAboutAction();
    void closeAboutSIMphotoDialog();

public:
    SIMphoto(QWidget *parent = 0);
    ~SIMphoto();

private:
    void createActions();
    void createMenus();
    void createToolBars();
    void createCentralWidget();
    void createZoom();
    void getInitPath(QString& initPath);
    void initImage(QString name);
    void saveInitPath(QString initPath);
    void closeEvent(QCloseEvent *closeEvent);
    void createOverview();
    void doOverviewMoveEvent(int x, int y);
    void createMarkingTool(int pointsCount);
    void disableOtherActions();
    void enableOtherActions();
    void movePoint(int moveX, int moveY);
    void keyPressEvent(QKeyEvent *event);
    void drawOverview();
    void getZoomArea(int x, int y);
    void placePointFromZoomArea(int x, int y);
    void createNetworkMethodDialog();
    void checkNetworkValidity(bool& nonzeroDistance, bool& triangleCondition, Matrix<double, 6, 1> d, int diag);
    void computeCoordsFromNetwork(Matrix<double, 6, 1> d, int diag);
    void showDeviationMessage(double dev, Matrix<double, 4, 2> coordsFromNet);
    void drawNetworkPath();
    void createDistMethodDialog();
    void drawDistLines();
    void createMeasureTool();
    void createTableOfPoints();
    void createSimilarTransDialog();
    void createDistInputDialog();
    void createScalebarDialog();
    void drawScalebar(int x, int y);
    void setScalebarValue(int &scaleWidth, double &scaleValue);
    void imageWasModified();
    void doSimilarTransformation(double scaleForTrans, double rotationAngle);
    void doAffineTransformation();
    void createPixelSizeDialog();
    void getCheckedPoints();
    void estimateScale();
    void doColinearTransformation();
    void checkPointPosition();
    void createCheckPointDialog();
    void closeCheckPointDialog();
    void showObjectCoords(int x, int y);
    void CoeffForObjCrd();
    void createDistortionDialog();
    void doDistortionReduction();
    void showExistingTable(QList<QString> residualXList, QList<QString> residualYList, Matrix<int, 9, 1> usePoint);
    void getTransformCoefficients(Matrix<double, Eigen::Dynamic, 1> x1, Matrix<double, Eigen::Dynamic, 1> y1,
                                  Matrix<double, Eigen::Dynamic, 1> x2, Matrix<double, Eigen::Dynamic, 1> y2);

    bool maybeSave();
    bool eventFilter(QObject *obj, QEvent *event);

    Matrix<double, 4, 2> putY_axesToDist4(Matrix<double, 4, 2> system, double dist4);
    Matrix<double, 4, 2> distanceIntersection(double X_Axes, double dA_AboveX, double dB_AboveX,
                                              double dA_BelowX,  double dB_BelowX, bool isDist6,
                                              bool& angleIsAcceptable);

    double maxPositionDeviation(Matrix<double, 4, 2> system1, Matrix<double, 4, 2> system2);
    double specifyQuadrant(int topX, int topY, int bottomX, int bottomY);
    double distanceFromCoords(double XA, double YA, double XB, double YB);

    QSize newImageSizeForSimTrans(double transScale, double transAngle);
    QSize newImageSizeForColTrans();

    QPainterPath overviewPath(double overviewScale);

    QMenu *imageMenu;
    QMenu *editMenu;
    QMenu *photomapMenu;
    QMenu *functionsMenu;
    QMenu *viewMenu;
    QMenu *zoomMenu;
    QMenu *helpMenu;

    QAction *openAct;
    QAction *saveAct;
    QAction *printAct;
    QAction *exitAct;
    QAction *undoAct;
    QAction *controlPointsMethodAct;
    QAction *networkMethodAct;
    QAction *distMethodAct;
    QAction *similarTransAct;
    QAction *measureAct;
    QAction *distorsionRedAct;
    QAction *scalebarAct;
    QAction *loadTableAct;
    QAction *overviewAct;
    QAction *cursorColorAct;
    QAction *zoom3xAct;
    QAction *zoom5xAct;
    QAction *noZoomAct;
    QAction *aboutAct;

    QToolBar *imageToolBar;
    QToolBar *photomapToolBar;
    QToolBar *functionsToolBar;
    QToolBar *viewToolBar;

    QLabel *imageLabel;
    QLabel *zoomLabel;

    QScrollArea *imageScrollArea;

    QImage *image;
    QImage *zoomImage;

    QDockWidget *overviewDock;
    QDockWidget *markDock;
    QDockWidget *checkPointDock;

    QCursor crossCursor;

    QRect scalebarRect;

    bool blackCursorColor;
    bool zoomIsActive;
    bool finishScalebarAction;

    int zoomInt;
    int checkedPointsCount;
    int focusedPoint;

    double alphaAngle;
    double translationX;
    double translationY;
    double maxCoordinate;

    Matrix<int, 9, 1> pointIndex;
    Matrix<double, Eigen::Dynamic, 1> xx, yy, XX, YY;
    Matrix<double, Eigen::Dynamic, 1> coeff;
    Matrix<double, Eigen::Dynamic, 1> v;
    Matrix<double, 2, 2> cornersCoords;
    Matrix<double, 2, 2> previousCornersCoords;

    Overview *overviewLabel;
    MarkTool *markingTool;
    CursorColorDialog *cursorColorDialog;
    NetworkMethodDialog *networkMethodDialog;
    MemoryClass memory;
    DistMethodDialog *distMethodDialog;
    MeasureTool *measureTool;
    SimilarTransformDialog *similarTransDialog;
    TableOfPoints *tableOfPoints;
    ScalebarDialog *scalebarDialog;
    DistanceInputDialog *distInputDialog;
    PixelSizeDialog *pixelSizeDialog;
    CheckPointDialog *checkPointDialog;
    ClipImageDialog *clipImageDialog;
    MapLayoutDialog *mapLayoutDialog;
    DistorsionDialog *distorsionDialog;
    AboutDialog *aboutSIMphotoDialog;
};

#endif // SIMPHOTO_H
