/************************************************************************************
**    SIMphoto is an aplication based on single image photogrammetry. It is useful
**    for creating and printing a photomap. The software is designed for close
**    range photogrammtery. Outputs are usable in the field of heritage preservation.
**    Copyright (C) 2009 David Cizek
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**    How to contact me: cizekdav@gmail.com
**
***********************************************************************************/

#include "tableofpoints.h"

TableOfPoints::TableOfPoints(int pointCount, QWidget *parent)
    : QDialog(parent)
{
    table = new QTableWidget(this);

    Qt::WindowFlags flags;
    flags = Qt::Dialog;
    flags |= Qt::WindowTitleHint;
    flags |= Qt::CustomizeWindowHint;

    setWindowFlags(flags);
    setModal(true);
    setWindowTitle(tr("Planar Projective Transformation Table"));

    table->setColumnCount(8);
    table->setMinimumSize(560, 200);
    table->verticalHeader()->setVisible(false);
    table->setHorizontalHeaderLabels(QStringList() << tr("Use")
                                                   << tr("Point")
                                                   << tr("X [m]")
                                                   << tr("Y [m]")
                                                   << tr("x [p]")
                                                   << tr("y [p]")
                                                   << tr("Residual X [m]")
                                                   << tr("Residual Y [m]"));

    table->horizontalHeaderItem(6)->setFont(QFont("Arial", 9, QFont::Bold));
    table->horizontalHeaderItem(7)->setFont(QFont("Arial", 9, QFont::Bold));

    showResidualsButton = new QPushButton(tr("Show Residuals"), this);
    transformButton = new QPushButton(tr("Transform"), this);
    saveTableButton = new QPushButton(tr("Save Table"), this);
    loadCoordsButton = new QPushButton(tr("Load Coordinates"), this);
    skipButton = new QPushButton(tr("Skip To Marking"), this);

    QGridLayout *pointTableLayout = new QGridLayout;
    QHBoxLayout *buttonsLayout  = new QHBoxLayout;
    QSpacerItem *spacer = new QSpacerItem(50, 8, QSizePolicy::Expanding, QSizePolicy::Minimum);

    buttonsLayout->addWidget(saveTableButton);
    buttonsLayout->addWidget(loadCoordsButton);
    buttonsLayout->addSpacerItem(spacer);
    buttonsLayout->addWidget(skipButton);
    buttonsLayout->addWidget(showResidualsButton);
    buttonsLayout->addWidget(transformButton);

    pointTableLayout->addWidget(table, 1, 0);
    pointTableLayout->addLayout(buttonsLayout, 2, 0);

    setLayout(pointTableLayout);
    adjustSize();

    rowCount = pointCount;
}

void TableOfPoints::insertData(Matrix<double, 9, 2> objectXY, QList<QLabel*> imageXY, bool network)
{
    for(int i = 0; i < rowCount; i++)
    {
        usePointCheckBox = new QCheckBox;
        usePointCheckBox->setChecked(true);
        checkBoxList.append(usePointCheckBox);

        objectX = new QDoubleSpinBox;
        objectX->setRange(-1000.0, 1000.0);
        objectX->setDecimals(3);
        objectX->setValue(objectXY(i, 0));
        objectXList.append(objectX);

        objectY = new QDoubleSpinBox;
        objectY->setRange(-1000.0, 1000.0);
        objectY->setDecimals(3);
        objectY->setValue(objectXY(i, 1));
        objectYList.append(objectY);
    }

    showData(imageXY, network);
    enableButtons(network);
}

void TableOfPoints::showData(QList<QLabel*> imageXY, bool networkMethod)
{
    table->setRowCount(rowCount);

    for(int i = 0; i < rowCount; i++)
    {
        table->setCellWidget(i, 0, checkBoxList.at(i));

        item = new QTableWidgetItem;
        item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
        item->setData(Qt::DisplayRole, QString::number(i + 1));
        item->setTextAlignment(Qt::AlignHCenter);
        table->setItem(i, 1, item);

        if(networkMethod)
        {
            item = new QTableWidgetItem;
            item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
            item->setData(Qt::DisplayRole, objectXList.at(i)->value());
            item->setTextAlignment(Qt::AlignHCenter);
            table->setItem(i, 2, item);

            item = new QTableWidgetItem;
            item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
            item->setData(Qt::DisplayRole, objectYList.at(i)->value());
            item->setTextAlignment(Qt::AlignHCenter);
            table->setItem(i, 3, item);
        }
        else
        {
            table->setCellWidget(i, 2, objectXList.at(i));
            table->setCellWidget(i, 3, objectYList.at(i));
        }

        item = new QTableWidgetItem;
        item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
        item->setData(Qt::DisplayRole, imageXY.at(i)->x() + 11);
        item->setTextAlignment(Qt::AlignHCenter);
        table->setItem(i, 4, item);

        item = new QTableWidgetItem;
        item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
        item->setData(Qt::DisplayRole, imageXY.at(i)->y() + 43);
        item->setTextAlignment(Qt::AlignHCenter);
        table->setItem(i, 5, item);

        item = new QTableWidgetItem;
        item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
        item->setTextAlignment(Qt::AlignHCenter);
        table->setItem(i, 6, item);

        item = new QTableWidgetItem;
        item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
        item->setTextAlignment(Qt::AlignHCenter);
        table->setItem(i, 7, item);
    }

    table->resizeColumnToContents(0);
    table->resizeColumnToContents(1);
    table->resizeColumnToContents(4);
    table->resizeColumnToContents(5);
    table->resizeColumnToContents(6);
    table->resizeColumnToContents(7);

    table->horizontalHeader()->setStretchLastSection(true);
    table->verticalHeader()->setResizeMode(QHeaderView::ResizeToContents);
}

void TableOfPoints::enableButtons(bool networkMethod)
{
    if(networkMethod)
        loadCoordsButton->setEnabled(false);

    if(table->rowCount() == 4)
    {
        transformButton->setEnabled(true);
    }
    else
    {
        transformButton->setEnabled(false);
    }

    if(table->rowCount() > 4)
    {
        showResidualsButton->setEnabled(true);
    }
    else
    {
        showResidualsButton->setEnabled(false);
    }
}

QList<QCheckBox*> TableOfPoints::getCheckBoxList()
{
    return checkBoxList;
}

QTableWidget* TableOfPoints::getTable()
{
    return table;
}

QList<QDoubleSpinBox*> TableOfPoints::getObjectXList()
{
    return objectXList;
}

QList<QDoubleSpinBox*> TableOfPoints::getObjectYList()
{
    return objectYList;
}

QPushButton* TableOfPoints::getTransformButton()
{
    return transformButton;
}

QPushButton* TableOfPoints::getSkipButton()
{
    return skipButton;
}

QPushButton* TableOfPoints::getSaveTableButton()
{
    return saveTableButton;
}

QPushButton* TableOfPoints::getLoadCoordsButton()
{
    return loadCoordsButton;
}

QPushButton* TableOfPoints::getShowResidualsButton()
{
    return showResidualsButton;
}
